@@ -1,19 +1,12 @@
 from __future__ import annotations
 
-import sys
 from enum import IntEnum
-from typing import Any, Dict, NamedTuple, Tuple
+from typing import Any, Dict, Literal, NamedTuple, Tuple
 
 import numpy as np
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-
-    Prec = Literal["d", "f", "i"]
-    Dim = Literal[1, 2, 3]
-else:
-    Prec = str
-    Dim = int
+Prec = Literal["d", "f", "i"]
+Dim = Literal[1, 2, 3]
 
 
 class Shape(NamedTuple):
@@ -33,20 +26,26 @@ class Shape(NamedTuple):
 
 
 class Coordinates(NamedTuple):
+    # Store 3 1D coordinates arrays and one 'array_shape'
     x1: np.ndarray
     x2: np.ndarray
     x3: np.ndarray
+    array_shape: Shape  # the 3D shape that 1D arrays should be broadcasted to
 
     @property
     def shape(self) -> Shape:
         return Shape(len(self.x1), len(self.x2), len(self.x3))
 
+    @property
+    def arrays(self) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
+        return self.x1, self.x2, self.x3
+
     def padded(self) -> Coordinates:
-        arrs = list(self)
-        for i, arr in enumerate(arrs):
+        arrs = [_.copy() for _ in self.arrays]
+        for i, arr in enumerate(self.arrays):
             if arr.size == 1:
                 arrs[i] = np.array((arr[0], arr[0] + 1))
-        return Coordinates(*arrs)
+        return Coordinates(arrs[0], arrs[1], arrs[2], self.array_shape)
 
 
 # map field name to numpy array init data:
